"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.find = function (result, document) {
    var element = document.querySelector(result.s);
    if (!element) {
        throw new Error('Unable to find element with selector: ' + result.s);
    }
    return element.childNodes[result.c];
};
exports.generateSelector = function (node, relativeTo) {
    var currentNode = node;
    var tagNames = [];
    var textNodeIndex = 0;
    if (node.parentNode) {
        textNodeIndex = exports.childNodeIndexOf(node.parentNode, node);
        while (currentNode) {
            var tagName = currentNode.tagName;
            if (tagName) {
                var nthIndex = exports.computedNthIndex(currentNode);
                var selector = tagName;
                if (nthIndex > 1) {
                    selector += ":nth-of-type(" + nthIndex + ")";
                }
                tagNames.push(selector);
            }
            currentNode = (currentNode.parentNode || currentNode.parentElement);
            if (currentNode == (relativeTo.parentNode || relativeTo.parentElement)) {
                break;
            }
        }
    }
    return { s: tagNames.reverse().join(">").toLowerCase(), c: textNodeIndex, o: 0 };
};
exports.childNodeIndexOf = function (parentNode, childNode) {
    var childNodes = parentNode.childNodes;
    var result = 0;
    for (var i = 0, l = childNodes.length; i < l; i++) {
        if (childNodes[i] === childNode) {
            result = i;
            break;
        }
    }
    return result;
};
exports.computedNthIndex = function (childElement) {
    var elementsWithSameTag = 0;
    var parent = (childElement.parentNode || childElement.parentElement);
    if (parent) {
        for (var i = 0, l = parent.childNodes.length; i < l; i++) {
            var currentHtmlElement = parent.childNodes[i];
            if (currentHtmlElement === childElement) {
                elementsWithSameTag++;
                break;
            }
            if (currentHtmlElement.tagName === childElement.tagName) {
                elementsWithSameTag++;
            }
        }
    }
    return elementsWithSameTag;
};
//# sourceMappingURL=HtmlElementSelectorGenerator.js.map