"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var RangeSerializer_1 = require("./RangeSerializer");
var RangeCompressor_1 = require("./RangeCompressor");
var RangeEncoder_1 = require("./RangeEncoder");
var Rangee = /** @class */ (function () {
    function Rangee(options) {
        var _this = this;
        this.onSerialization = function (callback) { return _this.serializationCallback = callback; };
        this.onCompression = function (callback) { return _this.compressionCallback = callback; };
        this.serializeAtomic = function (range) {
            var atomicRanges = _this.createAtomicRanges(range);
            var serialized = atomicRanges
                .map(function (range) { return RangeSerializer_1.serialize(range.cloneRange(), _this.options.document.body); })
                .map(function (serializedRange) { return JSON.stringify(serializedRange); })
                .join("|");
            _this.serializationCallback && _this.serializationCallback(serialized);
            var compressed = RangeCompressor_1.compress(serialized);
            _this.compressionCallback && _this.compressionCallback(compressed);
            var encoded = RangeEncoder_1.encode(compressed);
            return encoded;
        };
        this.deserilaizeAtomic = function (representation) {
            var decoded = RangeEncoder_1.decode(representation);
            var decompressed = RangeCompressor_1.decompress(decoded);
            var serializedRanges = decompressed
                .split("|")
                .map(function (decompressedRangeRepresentation) { return JSON.parse(decompressedRangeRepresentation); })
                .reverse()
                .map(function (serializedRange) { return RangeSerializer_1.deserialize(serializedRange, _this.options.document); });
            return serializedRanges;
        };
        this.serialize = function (range) {
            var serialized = RangeSerializer_1.serialize(range.cloneRange(), _this.options.document.body);
            var serializedStringified = JSON.stringify(serialized);
            var compressed = RangeCompressor_1.compress(serializedStringified);
            var encoded = RangeEncoder_1.encode(compressed);
            return encoded;
        };
        this.deserialize = function (serialized) {
            var decoded = RangeEncoder_1.decode(serialized);
            var decompressed = RangeCompressor_1.decompress(decoded);
            var decompressedParsed = JSON.parse(decompressed);
            var deserilaized = RangeSerializer_1.deserialize(decompressedParsed, _this.options.document);
            return deserilaized;
        };
        this.createAtomicRanges = function (range) {
            // text
            if (range.startContainer === range.endContainer && range.startContainer.nodeType === Node.TEXT_NODE) {
                return [range];
            }
            var documentAsAny = _this.options.document; // IE does not know the right spec signature for createTreeWalker
            // elements
            var treeWalker = documentAsAny.createTreeWalker(range.commonAncestorContainer, NodeFilter.SHOW_ALL, function (node) { return NodeFilter.FILTER_ACCEPT; }, false);
            var startFound = false;
            var endFound = false;
            var atomicRanges = [];
            var node;
            while (node = treeWalker.nextNode()) {
                if (node === range.startContainer) {
                    startFound = true;
                }
                if (node.nodeType === Node.TEXT_NODE && startFound && !endFound && node.textContent && node.textContent.trim().length > 0) {
                    var atomicRange = _this.options.document.createRange();
                    atomicRange.setStart(node, node === range.startContainer ? range.startOffset : 0);
                    atomicRange.setEnd(node, node === range.endContainer ? range.endOffset : node.length);
                    atomicRanges.push(atomicRange);
                }
                if (node === range.endContainer) {
                    endFound = true;
                }
            }
            return atomicRanges;
        };
        this.options = __assign({}, options);
        this.serializationCallback = null;
        this.compressionCallback = null;
    }
    return Rangee;
}());
exports.Rangee = Rangee;
//# sourceMappingURL=index.js.map